////////////////
// EWIN2.CPP: 
// Sample MFC program demonstrating how to write CWnd::EnumChildWindows.
//

#include "stdafx.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

//////////////////
// Main application class.
//
class CMyApp : public CWinApp {
public:
	CMyApp();
	virtual BOOL InitInstance();
};

//////////////////
// Similar to WNDENUMPROC, only for CWnd member function.
// Declares CWNDENUMPROC as a pointer to a member function
// of CWnd that takes an LPARAM and returns a void.
//
typedef void (CWnd::*CWNDENUMPROC)(LPARAM);

//////////////////
// An enumerable CWnd class. If MFC had implemented
// CWnd::EnumChildWindows, this is what it would look like.
//
class CEnumerableWnd : public CWnd {
public:
	void EnumChildWindows(CWNDENUMPROC, LPARAM=0);
};

//////////////////
// Some arbitrary application window class. 
// Must be derived from CEnumerableWnd to use the Enum feature.
//
class CMyWnd : public CEnumerableWnd {
public:
	void MyEnumFn(LPARAM=0);	// An enumerator function. 
										// Could have more than one.
};

CMyApp theApp;

CMyApp::CMyApp()
{
	
}

//////////////////
// Initialize app: 
// Do enumeration on child windows of desktop
//
BOOL CMyApp::InitInstance()
{
	// Get desktop (topmost) window. Note required cast.
	CEnumerableWnd* pwndDesktop = (CEnumerableWnd*)CWnd::GetDesktopWindow();
	ASSERT(pwndDesktop);

	// Enumerate using MyEnumFn. 
	// Note & syntax and another required cast
	//
	pwndDesktop->EnumChildWindows((CWNDENUMPROC)&CMyWnd::MyEnumFn);
	return FALSE;				// return FALSE to quit app
}

//////////////////
// This is the implementation of EnumChildWindows for MFC.
// Invoke caller-supplied function for each child window.
//
void CEnumerableWnd::EnumChildWindows(CWNDENUMPROC pEnumFn, LPARAM lp)
{
	for (CWnd* pwnd = GetTopWindow();
		  pwnd;
		  pwnd = pwnd->GetNextWindow()) 
	{
		(pwnd->*pEnumFn)(lp);		// call enumerator function w/arg
	}
}

//////////////////
// Application-specific enumerator function.
// For CMyWnd, it does the same thing as in WINNAV; namely,
// it writes the title of the window to the Debug output.
//
void CMyWnd::MyEnumFn(LPARAM lp)
{
	char buf[80];
	GetWindowText(buf, sizeof(buf));
	TRACE1("Window: %s\n", buf);
}
